/*
 *  Real valued 2-dimensional vector class
 *
 *  Esa Hyyti, 2004-2005
 */

package elib;

import java.awt.Point;
import java.lang.Math;

public class Loc
{
    public Loc( )                      { x = 0;   y = 0;   }
    public Loc( Point p )              { x = p.x; y = p.y; }
    public Loc( double x_, double y_ ) { x = x_;  y = y_;  }
    public Loc( Loc B )                { set( B );         }

    /*********************************************************/

    public double x;  /* member variables */
    public double y;

    /*********************************************************/

    public void set( Loc B )                { x = B.x;  y = B.y;  }
    public void set( double x_, double y_ ) { x = x_;   y = y_;   }
    public void add( Loc B )                { x += B.x; y += B.y; }
    public void add( double x_, double y_ ) { x += x_;  y += y_;  }
    public void sub( Loc B )                { x -= B.x; y -= B.y; }
    public void sub( double x_, double y_ ) { x -= x_;  y -= y_;  }
    public void mul( double q )             { x *= q;   y *= q;   }

    public double len( )                    { return Math.sqrt( x*x + y*y ); }
    public double len2( )                   { return x*x + y*y;              }
    public double dist( Loc B )             { return Math.sqrt( dist2( B ) ); }
    
    public double dist2( Loc B )            
    {
        Loc D = new Loc( this ); 
        D.sub( B ); 
        return D.len2( );
    }

    public boolean equals( Loc B )
    {
        return (x==B.x && y==B.y ? true : false);
    }
    
    // returns unit vector to the same direction
    public boolean normalize( )  
    {
        if ( x==0 && y==0 )
            return false;
        
        double q = 1.0/len( );
        x *= q;
        y *= q;
        return true;
    }

    /*
     *  Returns the angle the vector is pointing to, [0,2pi),
     *  if zero vector the method returns 0.
     */
    public double angle( )    
    {
        double rad = Math.atan2( y, x );
        if( rad < 0.0 )
            rad += 2.0*Math.PI;
        return rad;
    }

    public double angle( Loc origo )
    {
        double rad = Math.atan2( y-origo.y, x-origo.x );
        if( rad < 0.0 )
            rad += 2*Math.PI;
        return rad;
    }
    
    /*
     *  Basic rotations
     */
    public void rotate_90( )  { double tmp=x;  x = -y;  y =  tmp; }
    public void rotate_180( ) {                x = -x;  y =   -y; }
    public void rotate_270( ) { double tmp=x;  x =  y;  y = -tmp; }
    public void reverse( )    { rotate_180(); }        
    public void rotate( double a ) 
    { 
        double cosa = Math.cos( a );
        double sina = Math.sin( a );
        double nx   = cosa * x - sina * y;
        double ny   = cosa * y + sina * x;
        x = nx;
        y = ny;
    }
}
